/*****************************************************
Chip type               : ATtiny13A
AVR Core Clock frequency: 9,600000 MHz
CVAVR 2.04
*****************************************************/

#include <tiny13a.h>

#define Rb_upr 27000 //Ohms
#define Rb_lwr 2000   //Ohms
#define Ub_max_adc ((110*(Rb_upr+Rb_lwr))/Rb_lwr) //=U_max*100, 15,95V*100

#define HIL PORTB.1
#define LOL PORTB.0
#define HIR PORTB.3
#define LOR PORTB.4

#define ADC_itn 8

unsigned char tmr0, tmr1, ADC1;
bit LoBat;
flash unsigned char s_table [9]={7,14,20,23,24,23,20,14,7};

void port_cfg()
{
PORTB   =0b11110001;
DDRB    =0b00011011;
}
void t0_cfg()
{
TCCR0B=0x02;
TCNT0=207;
TIMSK0=0x02;
}
void adc_cfg()
{
 ADMUX|=1<<REFS0|1<<ADLAR|1;
 ADCSRA|=1<<ADEN|1<<ADSC|7;
}

void adc_scan()
{
 static unsigned char ADC1_tmp [ADC_itn]; 
 unsigned int dtmp=0;
 unsigned char i = 0;
 static bit a1f=0;
 
 if (!a1f) {for(i=0;i<ADC_itn-1;i++){ADC1_tmp[i]=200;a1f=1;}}

if(!ADCSRA.ADSC)
{ 
  for (i=0; i<(ADC_itn-1); i++ ) {ADC1_tmp [i] = ADC1_tmp [i+1];}
  ADC1_tmp [(ADC_itn-1)] = ADCH;
  for (i=0; i<(ADC_itn); i++ ) { dtmp += ADC1_tmp [i];}
  ADC1 = dtmp/ADC_itn; 
  ADCSRA |= 1<<ADSC;
}
}
void bat_detect()
{
// unsigned char tmp;  
// tmp=((unsigned long)ADC1*Ub_max_adc/(0xff*10));
   
 if(ADC1>170){tmr1=250;}   
 else if (ADC1<50){LoBat=1;tmr0=250;}
 if(!tmr1) {LoBat=1;tmr0=250;} 
 if(!tmr0) {LoBat=0;}
}



interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
TCNT0=207; //25Hz 40us dt
static bit wave_pol;
static unsigned char tcnt, pcnt, i;

if(!LoBat)
{
if(wave_pol)
{
    if(pcnt)            
     {  
        if(tcnt<25)
        {
            if (tcnt<((25-s_table[pcnt-1])>>1)){HIL=LOL=0;}
            else if (tcnt==((25-s_table[pcnt-1])>>1)){HIL=0;LOL=1;}
            else if ((tcnt+1)<(((25-s_table[pcnt-1])>>1)+s_table[pcnt-1])){HIL=LOL=1;} 
            else if ((tcnt+1)==(((25-s_table[pcnt-1])>>1)+s_table[pcnt-1])){HIL=0;LOL=1;}
            else {HIL=LOL=0;}
        tcnt++; 
        }
        else
        {
        tcnt=0;
        pcnt--;
        }
    }
    else
    { 
    HIL=LOL=0;
    pcnt=9; 
    wave_pol=0; 
    if(tmr0)tmr0--;
    } 
}

else
{
    if(pcnt)            
     {  
        if(tcnt<25)
        {
            if (tcnt<((25-s_table[pcnt-1])>>1)){HIR=LOR=0;}
            else if (tcnt==((25-s_table[pcnt-1])>>1)){HIR=0;LOR=1;}
            else if ((tcnt+1)<(((25-s_table[pcnt-1])>>1)+s_table[pcnt-1])){HIR=LOR=1;} 
            else if ((tcnt+1)==(((25-s_table[pcnt-1])>>1)+s_table[pcnt-1])){HIR=0;LOR=1;}
            else {HIR=LOR=0;}
        tcnt++; 
        }
        else
        {
        tcnt=0;
        pcnt--;
        }
     }
    else
    { 
    HIR=LOR=0;
    pcnt=9;
    wave_pol=1;
    if(tmr1)tmr1--;
    } 
} 
#asm("wdr")
}
else
{
HIL=LOL=HIR=LOR=pcnt=tcnt=0;
if(i)i--;
else
{
 i=250;
 if(tmr0)tmr0--;
 if(tmr1)tmr1--;
}
#asm("wdr")
}
}
void main(void)
{
port_cfg();
t0_cfg();
adc_cfg();


{
// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif
// Watchdog Timer Prescaler: OSC/2k
#pragma optsize-
WDTCR=0x18;
WDTCR=0x08;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif
}
#asm("sei")

while (1)
      {
       adc_scan(); 
       bat_detect();
      }
}
